<?php

namespace Drupal\string_demo\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for testing string translations.
 */
class StringDemoTestForm extends FormBase {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Constructs a new StringDemoTestForm.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('language_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'string_demo_test_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Get all enabled languages.
    $languages = $this->languageManager->getLanguages();
    $language_options = [];
    foreach ($languages as $langcode => $language) {
      $language_options[$langcode] = $language->getName();
    }

    $form['form_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'form-wrapper'],
    ];

    $form['form_wrapper']['languages'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select languages to test'),
      '#options' => $language_options,
      '#default_value' => array_keys($language_options),
      '#required' => TRUE,
    ];

    // Built-in strings section.
    $form['form_wrapper']['builtin_strings'] = [
      '#type' => 'details',
      '#title' => $this->t('Built-in Strings'),
      '#open' => TRUE,
    ];

    $options = [
      'Content',
      'Pages',
      'drupal.node.content.label',
      'string_demo.dashboard.welcome_message.short',
      'string_demo.search.result.items_count',
    ];
    $form['form_wrapper']['builtin_strings']['string_ids'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select strings to test'),
      '#options' => array_combine($options, $options),
      '#default_value' => [
        'Content',
        'Pages',
        'drupal.node.content.label',
        'string_demo.dashboard.welcome_message.short',
        'string_demo.search.result.items_count',
      ],
    ];

    $form['form_wrapper']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Run test'),
      '#submit' => ['::submitForm'],
    ];

    // Add results if form was submitted.
    if ($form_state->isSubmitted() && !$form_state->hasAnyErrors()) {
      $form['results'] = $this->buildResults($form_state);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $languages = array_filter($form_state->getValue('languages'));
    if (empty($languages)) {
      $form_state->setErrorByName('languages', $this->t('Please select at least one language.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $form_state->setRebuild();
  }

  /**
   * Builds the results table.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The results table render array.
   */
  protected function buildResults(FormStateInterface $form_state): array {
    $languages_being_tested = array_filter($form_state->getValue('languages'));
    $results = $this->buildBuiltinResults($form_state, $languages_being_tested);

    // Create table with string ID as first column.
    $header = ['String ID', 'Count'];
    foreach ($languages_being_tested as $langcode) {
      $header[] = $langcode;
    }

    // First, organize the results by string_id and count.
    $organized_results = [];
    foreach ($results as $row) {
      $string_id = $row[0];
      $count = $row[2];
      $langcode = $row[4];
      $output = $row[5];

      if (!isset($organized_results[$string_id])) {
        $organized_results[$string_id] = [];
      }
      if (!isset($organized_results[$string_id][$count])) {
        $organized_results[$string_id][$count] = [];
      }
      $organized_results[$string_id][$count][$langcode] = $output;
    }

    // Then, create the table rows.
    $rows = [];
    foreach ($organized_results as $string_id => $counts) {
      foreach ($counts as $count => $translations) {
        $row_data = [
          $string_id,
          $count,
        ];

        // Add translations for each language.
        foreach ($languages_being_tested as $langcode) {
          $row_data[] = $translations[$langcode] ?? '-';
        }

        $rows[] = $row_data;
      }
    }

    // Sort rows by string_id and count.
    usort($rows, function ($a, $b) {
      $string_id_cmp = strcmp($a[0], $b[0]);
      if ($string_id_cmp !== 0) {
        return $string_id_cmp;
      }
      return $a[1] <=> $b[1];
    });

    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

  /**
   * Builds results for built-in strings.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array<string> $languages
   *   The languages to test.
   *
   * @return array<int, array<int, mixed>>
   *   The results array.
   */
  protected function buildBuiltinResults(FormStateInterface $form_state, array $languages): array {
    $string_ids = array_filter($form_state->getValue('string_ids'));
    $plural_numbers = $this->getPluralNumbers();
    $rows = [];

    foreach ($string_ids as $string_id) {
      if ($string_id === 'string_demo.search.result.items_count') {
        $this->preparePluralVariations($rows, $string_id, $languages, $plural_numbers);
      }
      else {
        $this->prepareVariations($rows, $string_id, $languages);
      }
    }

    return $rows;
  }

  /**
   * Gets an array of numbers to test plural forms.
   *
   * @return array<int>
   *   Array of numbers to test.
   */
  protected function getPluralNumbers(): array {
    $numbers = [];
    for ($count = 0; $count <= 30; $count++) {
      $numbers[] = $count;
    }
    $numbers[] = 40;
    $numbers[] = 50;
    $numbers[] = 60;
    $numbers[] = 70;
    $numbers[] = 80;
    $numbers[] = 90;
    $numbers[] = 100;
    for ($count = 100; $count <= 130; $count++) {
      $numbers[] = $count;
    }
    return $numbers;
  }

  /**
   * Prepare various cases for testing string.
   *
   * @param array<int, array<int, mixed>> $rows
   *   The rows array to modify.
   * @param string $string_id
   *   The string ID to test.
   * @param array<string> $languages_being_tested
   *   The languages to test.
   */
  protected function prepareVariations(array &$rows, string $string_id, array $languages_being_tested): void {
    $string_test = [
      'id' => $string_id,
      'params' => [
        '@count' => 1,
        '@name' => 'Faker',
      ],
    ];
    foreach ($languages_being_tested as $langcode) {
      foreach ([''] as $count) {
        $string_test['params']['@count'] = $count;
        $row = [
          $string_test['id'],
          't',
          $string_test['params']['@count'],
          $string_test['params']['@name'],
          $langcode,
          // phpcs:ignore Drupal.Semantics.FunctionT -- Used for testing.
          $this->t($string_test['id'], $string_test['params'], ['langcode' => $langcode]),
        ];
        $rows[] = $row;
      }
    }
  }

  /**
   * Prepare various cases for testing complex plural forms.
   *
   * @param array<int, array<int, mixed>> $rows
   *   The rows array to modify.
   * @param string $string_id
   *   The string ID to test.
   * @param array<string> $languages_being_tested
   *   The languages to test.
   * @param array<int> $plural_numbers
   *   The numbers to test plural forms with.
   */
  protected function preparePluralVariations(array &$rows, string $string_id, array $languages_being_tested, array $plural_numbers): void {
    $string_test = [
      'id' => $string_id,
      'params' => [
        '@count' => 1,
        '@name' => 'Faker',
      ],
    ];
    foreach ($plural_numbers as $count) {
      foreach ($languages_being_tested as $langcode) {
        $string_test['params']['@count'] = $count;
        $row = [
          $string_test['id'],
          'formatPlural',
          $string_test['params']['@count'],
          $string_test['params']['@name'],
          $langcode,
          $this->formatPlural($count, $string_test['id'], $string_test['id'], $string_test['params'], ['langcode' => $langcode]),
        ];
        $rows[] = $row;
      }
    }
  }

}
